# indextrack.mod   QLR2-AN-34-19
# AMPL coding: Sven Leyffer, University of Dundee, June 2000.
#
# Problem arising in the selection of a portfolio of securities to
# match the performance of a given index.
# Data are real data from Japanese banks.
#
# See "Algorithms for index tracking", J.W. Kwiatkowski, University 
# of Edinburgh.
#
# Data files: indextrack.dat (Select 8 holdings from 17 choices)

set securities;				# ... set of securities

param nsec default 6, integer;		# ... number of securities in holding
param mu {securities}; 			# ... market capitilization
param Q {securities,securities};	# ... covariance matrix
param lambda;				# ... scale of weights
param p {i in securities} 		# ... linear objective coefficient
      := sum{j in securities} Q[i,j]*mu[j] / lambda;

var x {securities} >= 0;      		# ... fraction of portfolio of holdings
var mvar {i in securities}   		# ... marginal variance
    = sum{j in securities} Q[i,j]*x[j];
var y {securities} binary;     		# ... binary for slecting holdings

minimize objx: sum{i in securities} x[i]*mvar[i] 
               - sum{i in securities} p[i]*x[i];

subject to

   # ... budget constraint
   budget: sum {i in securities} x[i] = 1;

   # ... choice of member in portfolio
   upbnd {i in securities}:  x[i] <= y[i];

   # ... restrict number of holdings in portfolio
   nhold: sum {i in securities} y[i] = nsec;



